;nyquist plug-in
;version 4
;type tool
;name "Rename Tracks"
;author "Steve Daulton"
;release 2.3.2
;copyright "Released under terms of the GNU General Public License version 2"

;; Released under terms of the GNU General Public License version 2:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
;;
;; For information about writing and modifying Nyquist plug-ins:
;; https://wiki.audacityteam.org/wiki/Nyquist_Plug-ins_Reference

;control text "Enter a list of track names in field below."
;control text "Each name must be in quotes."
;control text "Use # at start or end to include the track number."
;control names "Track names" string "" "\"Track #\""
;control loop "If more tracks than names" choice "Repeat List,Repeat First,Repeat Last,Stop Renaming" 0

(setf names (eval-string (format nil "'(~a)" names)))

(defun get-selected ()
  ;;; Return a list of selected track indices.
  (let ((tracks (aud-get-info "Tracks"))
        (track-ids ())
        (id 0))
    (dolist (trk tracks (reverse track-ids))
      (if (member '(selected 1) trk :test 'equal)
          (push id track-ids))
      (setf id (1+ id)))))


(defun make-name (trk-name id)
  ;;; Prefix or append track number to name if requested.
  ;;; Return name.
  (when (not (stringp trk-name))
    (throw 'err "Track names must be \"quoted\"."))
  (cond
    ((char= (char trk-name 0) #\#)
      (format nil "~a~a" (1+ id) (subseq trk-name 1)))
    ((char= (char trk-name (1- (length trk-name))) #\#)
      (format nil "~a~a"
              (subseq trk-name 0 (1- (length trk-name)))
              (1+ id)))
    (t trk-name)))
  

(defun get-names (track-ids)
  ;;; Return a list of (LIST track-id track-name)
  (let ((more-names t)
        (name-idx 0)
        (name-list ()))
    (dolist (id track-ids (reverse name-list))
      (push (list id (make-name (nth name-idx names) id))
            name-list)
      (when more-names
        (cond
          ((< name-idx (1- (length names)))
            (setf name-idx (1+ name-idx)))
          ((= loop 0) ;repeat names list
            (setf name-idx 0))
          ((= loop 1) ;repeat first
            (setf name-idx 0)
            (setf more-names nil))
          ((= loop 2) ;repeat last
            (setf more-names nil))
          (t (return (reverse name-list))))))))


(aud-import-commands)
(print names)

(catch 'err
  (if (> (length names) 0)
      (let ((rename-list (get-names (get-selected))))
        (dolist (trk rename-list)
          (aud-selecttracks :track (first trk) :trackcount 1 :mode "Set")
          (aud-settrack :name (second trk))))
      "Error.\nNo names entered."))
